"""
******************************************************************************
* @author  ARMCNC site:www.armcnc.net github:armcnc.github.io
******************************************************************************
"""

import sys
import subprocess
import linuxcnc
from .status import Status
from .command import Command
from .ini import Ini
from .error import Error

class CNC:

    def __init__(self, framework):
        self.framework = framework
        self.status = Status(self.framework)
        self.command = Command(self.framework)
        self.ini = Ini(self.framework)
        self.error = Error(self.framework)

    def start(self):
        linuxcnc_pid = subprocess.Popen(["pidof", "-x", "linuxcnc"], stdout=subprocess.PIPE)
        linuxcnc_pid_result = linuxcnc_pid.communicate()[0]
        if len(linuxcnc_pid_result) == 0:
            self.framework.utils.service.service_write({"command": "launch:status", "message": "", "data": False})
            sys.exit()
        self.framework.machine.is_alive = True
        self.framework.utils.service.service_write({"command": "launch:status", "message": "", "data": True})

    def message_callback(self, message):
        if message and message["command"] and message["command"] != "":
            if message["command"] == "desktop:control:device:estop":
                self.status.api.poll()
                if self.status.api.task_state == linuxcnc.STATE_ESTOP:
                    self.command.api.state(linuxcnc.STATE_ESTOP_RESET)
                else:
                    if self.status.api.task_state == linuxcnc.STATE_ESTOP_RESET or self.status.api.task_state == linuxcnc.STATE_ON or self.status.api.task_state == linuxcnc.STATE_OFF:
                        self.command.api.state(linuxcnc.STATE_ESTOP)
                self.command.api.wait_complete(0.5)

            if message["command"] == "desktop:control:device:start":
                self.status.api.poll()
                if self.status.api.task_state == linuxcnc.STATE_ESTOP:
                    return False
                if self.status.api.task_state == linuxcnc.STATE_ON:
                    self.command.api.state(linuxcnc.STATE_OFF)
                else:
                    if self.status.api.task_state == linuxcnc.STATE_OFF or self.status.api.task_state == linuxcnc.STATE_ESTOP_RESET:
                        self.command.api.state(linuxcnc.STATE_ON)
                self.command.api.wait_complete(0.5)

                print("read_line", self.status.api.read_line)
                print("linear_units", self.status.api.linear_units)
                print("paused", self.status.api.paused)
                print("estop", self.status.api.estop)
                print("enabled", self.status.api.enabled)
                print("state", self.status.api.state)
                print("interp_state", self.status.api.interp_state)
                print("task_state", self.status.api.task_state)
                print("homed", self.status.api.homed)
                print("--------------------")






"""
******************************************************************************
* @author  ARMCNC site:www.armcnc.net github:armcnc.github.io
******************************************************************************
"""

import threading
import linuxcnc

class Status:

    def __init__(self, framework):
        self.framework = framework
        self.linuxcnc = linuxcnc
        self.api = linuxcnc.stat()
        self.task = threading.Thread(name="status_task", target=self.task)
        self.task.daemon = True
        self.task.start()

    def task(self):
        while True:
            if self.framework.machine.is_alive:
                try:
                    self.api.poll()
                except linuxcnc.error as detail:
                    self.framework.utils.service.service_write({"command": "launch:machine:error", "message": detail, "data": False})
                self.framework.machine.stat = {}
                for x in dir(self.api):
                    if not x.startswith("_") and not callable(getattr(self.api, x)):
                        self.framework.machine.stat[x] = getattr(self.api, x)
                self.framework.utils.service.service_write({"command": "launch:machine:status", "message": "", "data": self.framework.machine.stat})
            self.framework.utils.set_sleep(0.05)



"""
******************************************************************************
* @author  ARMCNC site:www.armcnc.net github:armcnc.github.io
******************************************************************************
"""

import threading
import linuxcnc

class Machine:

    def __init__(self, framework):
        self.framework = framework
        self.user = "armcnc"
        self.is_alive = False
        self.stat = None
        self.config = None
        self.task = threading.Thread(name="machine_task", target=self.task)
        self.task.daemon = True
        self.task.start()

    def task(self):
        while True:
            if self.is_alive:
                self.get_config()
            self.framework.utils.set_sleep(0.5)

    def get_config(self):
        if self.stat and self.stat["ini_filename"] != "":
            config = linuxcnc.ini(self.stat["ini_filename"])
            if self.config is None:
                self.config = {}
            self.config["EMC"] = {
                "MACHINE": config.find("EMC", "MACHINE"),
                "DESCRIBE": config.find("EMC", "DESCRIBE"),
                "CONTROL_TYPE": config.find("EMC", "CONTROL_TYPE"),
                "DEBUG": config.find("EMC", "DEBUG"),
                "VERSION": config.find("EMC", "VERSION"),
            }
            self.config["DISPLAY"] = {
                "DISPLAY": config.find("DISPLAY", "DISPLAY"),
                "CYCLE_TIME": config.find("DISPLAY", "CYCLE_TIME"),
                "POSITION_OFFSET": config.find("DISPLAY", "POSITION_OFFSET"),
                "POSITION_FEEDBACK": config.find("DISPLAY", "POSITION_FEEDBACK"),
                "MAX_FEED_OVERRIDE": config.find("DISPLAY", "MAX_FEED_OVERRIDE"),
                "MAX_SPINDLE_OVERRIDE": config.find("DISPLAY", "MAX_SPINDLE_OVERRIDE"),
                "MAX_LINEAR_VELOCITY": config.find("DISPLAY", "MAX_LINEAR_VELOCITY"),
                "DEFAULT_LINEAR_VELOCITY": config.find("DISPLAY", "DEFAULT_LINEAR_VELOCITY"),
                "DEFAULT_SPINDLE_SPEED": config.find("DISPLAY", "DEFAULT_SPINDLE_SPEED"),
                "PROGRAM_PREFIX": config.find("DISPLAY", "PROGRAM_PREFIX"),
                "INCREMENTS": config.find("DISPLAY", "INCREMENTS"),
            }
            # self.framework.utils.service.service_write({"command": "launch:machine:config", "message": "", "data": self.config})
